/*****************************************************************************
 * $Id: gfx_funcs2.hh,v 1.8 2006/08/01 23:14:02 devel Exp $
 * @file
 * @author	Jim E. Brooks  http://www.jimbrooks.org
 * @brief	Various small functions: function macros, inline functions, template functions.
 * @remarks	Include this after gfx_types.hh.
 *          Writing "inline" in template functions does increase speed (gcc 3.4).
 *//*
 * LEGAL:	COPYRIGHT (C) 2004 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE (GPL).
 *****************************************************************************/

#ifndef GFX_FUNCS2_HH
#define GFX_FUNCS2_HH 1

namespace gfx {

////////////////////////////////////////////////////////////////////////////////
/////////////////////////// function macros ////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

// Compare degrees/radians while ignoring sign.
#define IF_DEG_GT( DEG1, DEG2 ) (ABS(DEG1) > ABS(DEG2))
#define IF_DEG_LT( DEG1, DEG2 ) (ABS(DEG1) < ABS(DEG2))
#define IF_RAD_GT( RAD1, RAD2 ) (ABS(RAD1) > ABS(RAD2))
#define IF_RAD_LT( RAD1, RAD2 ) (ABS(RAD1) < ABS(RAD2))

////////////////////////////////////////////////////////////////////////////////
/////////////////////////// inline functions ///////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

/// --deprecated--
/// A safer string copy that ensures the destination string isn't overrun
/// and that it'll be terminated by a null char.
static inline void
STRNCPY( char* dest, const char* src, uint destLen /* length of dest (not src) */ )
{
	if ( dest != NULL && src != NULL && destLen >= 1 && destLen < GFX_MAX_STR_LEN )
	{
		strncpy( dest, src, destLen );
		dest[destLen - 1] = '\0';		// destLen must be >= 1
	}
#if DEBUG
	else { ASSERT(0); }
#endif
}

/*****************************************************************************
 * Clamp value to 0xFF.
 * ClampFF(uint) is the fastest.
 *****************************************************************************/
static inline uint8
ClampFF( uint i )
{
	// gcc 4.x does inline an optimal x86 CMOV instruction.
	if ( EX(i < 0xff) )
		return uint8(i);
	else
		return 0xff;
}
static inline uint8
ClampFF( int i )
{
	if ( i < 0 )
		return 0;
	else if ( i > 0xff )
		return 0xff;
	else
		return uint8(i);
}
static inline uint8
ClampFF( fp f )
{
    if ( f >= 1.0 )
        return 0xff;
    else if ( f < 0.0 )
        return 0;
    else
        return uint8( f * 256.0 );
}

/*****************************************************************************
 * Clamp float to within 0.0,...,1.0.
 *****************************************************************************/
static inline fp
Clamp1( fp f )
{
	if ( UX(f < 0.0) )
		return 0.0;
	else if ( UX(f > 1.0) )
		return 1.0;
	else
		return f;
}
static inline fp
Clamp( fp f, fp low = 0.0, fp high = 1.0 )
{
	if ( UX( f < low ) )
		return low;
	else if ( UX( f > high ) )
		return high;
	else
		return f;
}

////////////////////////////////////////////////////////////////////////////////
///////////////////////// template functions ///////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

} // namespace gfx

#endif // GFX_FUNCS2_HH
